// Copyright: Michael Moerz 2000
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License with
// the Debian GNU/Linux distribution in file /usr/share/doc/common-licenses/GPL
// if not, write to the Free Software Foundation, Inc., 59 Temple Place,
// Suite 330, Boston, MA  02111-1307  USA


#include <stdlib.h>
#include "dlist.h"

/* create a new dlist node containing data */
DList* 
dlist_new(void* data) {
  DList* new = malloc( sizeof(DList) * 1 );
  
  new->next = NULL;
  new->prev = NULL;
  new->data = data;

  return new;
}

/* concatenate two lists */
void
dlist_cat(DList* tail_l1, DList* head_l2){
  tail_l1->next = head_l2;
  head_l2->prev = tail_l1; 
}

/* to shorten things a bit in some cases */
DList*
dlist_insertnew(DList** tail, void* data) {
  DList* new = dlist_new( data );
  return dlist_insert( tail, new );
}

/* can also be used to append to a list */
DList*
dlist_insert(DList** tail, DList* new) {
  if ( (*tail) ) {
    new->next = (*tail)->next;
    (*tail)->next = new;
    new->prev = (*tail);
  }     

  if ( new->next )  // if next is defined, then set its prev value correctly
    new->next->prev = new;
  else
    (*tail) = new;
  
  return new;
}

/* retrieves the head of a list */
DList* 
dlist_head(DList* head){
  DList* old;

  while( head ) {
    old = head;
    head = head->prev;
  }
  
  return old;
}

/* retrieves the tail of a list */
DList* 
dlist_tail(DList* head){
  DList* old;

  while( head ) {
    old = head;
    head = head->next;
  }
  
  return old;
}

/* returns the length of the list */
size_t 
dlist_length(DList* head){
  size_t n;

  for (n = 0; head; ++n)
    head = head->next;

  return n;
}

/* removes an entry from the list, and returns it so that it might
   be destroyed properly */
DList*
dlist_remove(DList** head, DList** tail, DList* entry) {
  if (entry != (*head)) {  // when not head
    if (entry->prev)       // and a previous entry exists
      entry->prev->next = entry->next; // tell the previous our next entry
  } else
    (*head) = entry->next; // otherwise next entry is head
        
  if (entry != (*tail)) {  // when not tail
    if (entry->next)       // and next entry exists
      entry->next->prev = entry->prev; // tell the next entry our previous one
  } else
    (*tail) = entry->prev; // otherwise previous is tail

  return entry;
}

